indexing

	description:

		"Gobo Eiffel Yacc: syntactical analyzer generator"

	copyright: "Copyright (c) 1999-2001, Eric Bezault and others"
	license: "Eiffel Forum License v1 (see forum.txt)"
	date: "$Date: 2002/07/13 13:40:46 $"
	revision: "$Revision: 1.14 $"

class GEYACC

inherit

	GEYACC_VERSION

	KL_SHARED_EXCEPTIONS
	KL_SHARED_ARGUMENTS
	KL_SHARED_STANDARD_FILES

	KL_IMPORTED_STRING_ROUTINES

creation

	execute

feature -- Processing

	execute is
			-- Start 'geyacc' execution.
		local
			fsm: PR_FSM
			parser_generator: PR_PARSER_GENERATOR
			out_file, token_file: KL_TEXT_OUTPUT_FILE
			cannot_write: UT_CANNOT_WRITE_TO_FILE_ERROR
			tokens_needed: BOOLEAN
			verbose_file: KL_TEXT_OUTPUT_FILE
		do
			if False then resurrect_code end

			Arguments.set_program_name ("geyacc")
			!! error_handler.make_standard
			read_command_line
			parse_input_file
			if grammar /= Void then
				if verbose_filename /= Void then
						-- Verbose mode.
					!! verbose_file.make (verbose_filename)
					verbose_file.open_write
					if verbose_file.is_open_write then
						grammar.reduce_verbose (error_handler, verbose_file)
						grammar.set_nullable
						!! fsm.make (grammar)
						fsm.resolve_conflicts_verbose (error_handler, verbose_file)
						fsm.print_machine (verbose_file)
						verbose_file.close
					else
						!! cannot_write.make (verbose_filename)
						error_handler.report_error (cannot_write)
						Exceptions.die (1)
					end
				else
					grammar.reduce (error_handler)
					grammar.set_nullable
					!! fsm.make (grammar)
					fsm.resolve_conflicts (error_handler)
				end
				!! parser_generator.make (fsm)
				if input_filename /= Void then
					parser_generator.set_input_filename (input_filename)
				end
				if token_classname /= Void then
						-- Print class text with token code constants.
					!! token_file.make (token_filename)
					token_file.open_write
					if token_file.is_open_write then
						parser_generator.print_token_class (token_classname, Version_number, token_file)
						token_file.close
					else
						!! cannot_write.make (token_filename)
						error_handler.report_error (cannot_write)
						Exceptions.die (1)
					end
				else
					tokens_needed := True
				end
				if output_filename /= Void then
					!! out_file.make (output_filename)
					out_file.open_write
					if out_file.is_open_write then
						parser_generator.print_parser (tokens_needed, actions_separated, out_file)
						out_file.close
					else
						!! cannot_write.make (output_filename)
						error_handler.report_error (cannot_write)
						Exceptions.die (1)
					end
				else
					parser_generator.print_parser (tokens_needed, actions_separated, std.output)
				end
			end
		end

	parse_input_file is
			-- Parse input file.
		local
			parser: PR_YACC_PARSER
			a_file: KL_TEXT_INPUT_FILE
			cannot_read: UT_CANNOT_READ_FILE_ERROR
		do
			!! parser.make (error_handler)
			if input_filename /= Void then
				!! a_file.make (input_filename)
				a_file.open_read
				if a_file.is_open_read then
					parser.parse_file (a_file)
					a_file.close
				else
					!! cannot_read.make (input_filename)
					error_handler.report_error (cannot_read)
					Exceptions.die (1)
				end
			else
				parser.parse_file (std.input)
			end
			if not parser.successful then
				Exceptions.die (1)
			else
				grammar := parser.last_grammar
			end
		end

	read_command_line is
			-- Read command line arguments.
		local
			i, nb: INTEGER
			arg: STRING
		do
			nb := Arguments.argument_count
			from i := 1 until i > nb loop
				arg := Arguments.argument (i)
				if arg.is_equal ("--version") or arg.is_equal ("-V") then
					report_version_number
				elseif arg.is_equal ("--help") or arg.is_equal ("-h") or arg.is_equal ("-?") then
					report_usage_message
				elseif arg.is_equal ("-t") or arg.is_equal ("-d") then
					i := i + 1
					if i > nb then
						report_usage_error
					else
						token_classname := STRING_.to_upper (Arguments.argument (i))
						if token_filename = Void then
							token_filename := STRING_.to_lower (token_classname)
							token_filename.append_string (Eiffel_extension)
						end
					end
				elseif arg.count > 9 and then arg.substring (1, 9).is_equal ("--tokens=") then
					token_classname := STRING_.to_upper (arg.substring (10, arg.count))
					if token_filename = Void then
						token_filename := STRING_.to_lower (token_classname)
						token_filename.append_string (Eiffel_extension)
					end
				elseif arg.count > 10 and then arg.substring (1, 10).is_equal ("--defines=") then
					token_classname := STRING_.to_upper (arg.substring (11, arg.count))
					if token_filename = Void then
						token_filename := STRING_.to_lower (token_classname)
						token_filename.append_string (Eiffel_extension)
					end
				elseif arg.is_equal ("-k") then
					i := i + 1
					if i > nb then
						report_usage_error
					else
						token_filename := Arguments.argument (i)
					end
				elseif arg.count > 14 and then arg.substring (1, 14).is_equal ("--tokens-file=") then
					token_filename := arg.substring (15, arg.count)
				elseif arg.is_equal ("-o") then
					i := i + 1
					if i > nb then
						report_usage_error
					else
						output_filename := Arguments.argument (i)
					end
				elseif arg.count > 14 and then arg.substring (1, 14).is_equal ("--output-file=") then
					output_filename := arg.substring (15, arg.count)
				elseif arg.is_equal ("-v") then
					i := i + 1
					if i > nb then
						report_usage_error
					else
						verbose_filename := Arguments.argument (i)
					end
				elseif arg.is_equal ("-x") then
					actions_separated := True
				elseif arg.count > 10 and then arg.substring (1, 10).is_equal ("--verbose=") then
					verbose_filename := arg.substring (11, arg.count)
				elseif i = nb then
					input_filename := arg
				else
					report_usage_error
				end
				i := i + 1
			end
			if input_filename = Void then
				report_usage_error
			end
		end

feature -- Access

	input_filename: STRING
	output_filename: STRING
	token_classname: STRING
	token_filename: STRING
	verbose_filename: STRING
	actions_separated: BOOLEAN
			-- Command line arguments

	grammar: PR_GRAMMAR
			-- Grammar description

	error_handler: UT_ERROR_HANDLER
			-- Error handler

feature -- Constants

	Eiffel_extension: STRING is ".e"

feature {NONE} -- Error handling

	report_usage_error is
			-- Report usage error and then terminate
			-- with exit status 1.
		do
			error_handler.report_error (Usage_message)
			Exceptions.die (1)
		end

	report_usage_message is
			-- Report usage message and exit.
		do
			error_handler.report_info (Usage_message)
			Exceptions.die (0)
		end

	report_version_number is
			-- Report version number and exit.
		local
			a_message: UT_VERSION_NUMBER
		do
			!! a_message.make (Version_number)
			error_handler.report_info (a_message)
			Exceptions.die (0)
		end

	Usage_message: UT_USAGE_MESSAGE is
			-- Geyacc usage message
		once
			!! Result.make ("[-hxV?][-t classname][-k filename][-v filename][-o filename] filename")
		ensure
			usage_message_not_void: Result /= Void
		end

feature {NONE} -- Implementation

	resurrect_code is
			-- Make sure that SmallEiffel does not complain about possible
			-- "calls on a Void target in the living Eiffel code".
		local
			et1: KL_EQUALITY_TESTER [PR_RULE]
			et2: KL_EQUALITY_TESTER [PR_TOKEN]
			et3: KL_EQUALITY_TESTER [PR_VARIABLE]
			et4: KL_EQUALITY_TESTER [PR_TYPE]
			et5: KL_EQUALITY_TESTER [INTEGER]
			et6: KL_EQUALITY_TESTER [STRING]
			et7: KL_EQUALITY_TESTER [PR_TRANSITION]
			et8: KL_EQUALITY_TESTER [PR_REDUCTION]
			et9: KL_EQUALITY_TESTER [PR_STATE]
			et10: KL_EQUALITY_TESTER [PR_PORTION]
			et11: KL_EQUALITY_TESTER [PR_CONFLICT]
			et12: KL_EQUALITY_TESTER [DS_ARRAYED_LIST [PR_VARIABLE]]
			et13: KL_EQUALITY_TESTER [DS_ARRAYED_LIST [PR_STATE]]
		do
			!! et1
			!! et2
			!! et3
			!! et4
			!! et5
			!! et6
			!! et7
			!! et8
			!! et9
			!! et10
			!! et11
			!! et12
			!! et13
		end

invariant

	error_handler_not_void: error_handler /= Void
	token_filename: token_classname /= Void implies token_filename /= Void

end
